@@ -6,7 +6,7 @@ import logging
 from collections import Counter
 from datetime import datetime
 from pathlib import Path
-from typing import Any, Dict, List
+from typing import Any, Dict, Iterable, List
 from urllib.parse import urlencode, urljoin
 
 import frictionless
@@ -351,6 +351,16 @@ def get_badge_url_and_message(badge):
     return (badge_url, msg)
 
 
+def iter_task_errors(report: dict, code_set: set[str] = None) -> Iterable[dict]:
+    """Iterate on errors that prevent optimal validation."""
+    if report["tasks"]:
+        yield from (
+            err
+            for err in report["tasks"][0]["errors"]
+            if code_set is None or err["code"] in code_set
+        )
+
+
 def validate(schema_instance: SchemaInstance, validata_resource: ValidataResource):
     """Validate source and display report."""
 
@@ -390,28 +400,49 @@ def validate(schema_instance: SchemaInstance, validata_resource: ValidataResourc
         if badge_stats:
             badge_url, badge_msg = get_badge_url_and_message(badge_stats)
 
-    # Non table errors
-    if validata_core_report["errors"]:
-        non_table_error = validata_core_report["errors"][0]
-        msg = non_table_error.message
-        log.error(msg)
-        ui_error_msg = f"Une erreur est survenue durant la validation : {msg}"
-        if msg.startswith("The validation task has an error"):
-            ui_error_msg = (
-                "La validation ne peut s'effectuer sur ce fichier, "
-                "celui-ci n'est pas compatible avec le schéma."
+    schema_errors = [
+        err["code"] == "schema-error" for err in validata_core_report["errors"]
+    ]
+    if schema_errors:
+        log.error(schema_errors[0])
+        flash_error(
+            (
+                "Le schéma n'est pas valide selon la spécification TableSchema."
+                "Validation annulée."
             )
+        )
+        return redirect(
+            compute_validation_form_url(schema_instance.request_parameters())
+        )
+
+    check_errors = list(iter_task_errors(validata_core_report, {"check-error"}))
+    if check_errors:
+        ui_error_msg = (
+            'Erreur de schéma dans la section "custom_checks" : '
+            f"{check_errors[0]['note']}"
+        )
+        flash_error(ui_error_msg)
+        return redirect(
+            compute_validation_form_url(schema_instance.request_parameters())
+        )
+
+    task_errors = list(iter_task_errors(validata_core_report))
+    if task_errors:
+        err_msg = task_errors[0]["note"]
+        log.error(err_msg)
+        ui_error_msg = (
+            f"Une erreur est survenue durant la validation : {err_msg!r}."
+            " Pour plus d'information, consultez la FAQ."
+        )
         flash_error(ui_error_msg)
         return redirect(
             compute_validation_form_url(schema_instance.request_parameters())
         )
 
     # Source error
-    source_errors = [
-        err
-        for err in validata_core_report["tasks"][0]["errors"]
-        if err["code"] in {"source-error", "unknown-csv-dialect"}
-    ]
+    source_errors = list(
+        iter_task_errors(validata_core_report, {"source-error", "unkwnown-csv-dialect"})
+    )
     if source_errors:
         source_error = source_errors[0]
         msg = (
